import unittest

import xml.etree.ElementTree as ET

from ast_parser.python_unparser import PythonUnparser


class TestPythonUnparseCode(unittest.TestCase):

    def test_unparse_variable_regular(self):
        input = '<variable mut="var" type="int" name="a" ref_name="_lsiyjvtbvnpmlml"/>'
        parser = PythonUnparser(ET.fromstring(input), True)
        parser.source = ""
        parser.unparse_node(parser.xml)
        self.assertIsNotNone(parser.source)
        self.assertEqual("a", parser.source)

    def test_unparse_variable_declaration(self):
        input = '<variable mut="var" type="int" name="a" ref_name="_lsiyjvtbvnpmlml"/>'
        parser = PythonUnparser(ET.fromstring(input), True)
        parser.source = ""
        parser.unparse_variable(parser.xml, True)
        self.assertIsNotNone(parser.source)
        self.assertEqual("a: int", parser.source)


    def test_unparse_rhs_single(self):
        input = '<rhs><literal type="int" value="1" /></rhs>'
        parser = PythonUnparser(ET.fromstring(input), True)
        parser.source = ""
        parser.unparse_node(parser.xml)
        self.assertIsNotNone(parser.source)
        self.assertEqual("1", parser.source)

    def test_unparse_declaration(self):
        input = '<declaration><variable mut="var" type="int" name="a" ref_name="_lsiyjvtbvnpmlml"/><rhs><literal type="int" value="1" /></rhs></declaration>'
        parser = PythonUnparser(ET.fromstring(input), True)
        parser.source = ""
        parser.unparse_node(parser.xml)
        self.assertIsNotNone(parser.source)
        self.assertEqual("a: int = 1\n", parser.source)

    def test_unparse_stream(self):
        input = ' <stream type="std_output">                <operator op="*" type="unset">                    <lhs>                        <variable mut="var" type="int" name="a" ref_name="_lsiyjvtbvnpmlml"/>                    </lhs>                    <rhs>                        <literal type="int" value="42" />                    </rhs>                </operator></stream>'
        parser = PythonUnparser(ET.fromstring(input), True)
        parser.source = ""
        parser.unparse_node(parser.xml)
        self.assertIsNotNone(parser.source)
        self.assertEqual("print(a * 42, end='')\n", parser.source)

    def test_unparse_block(self):
        input = '<block> <declaration> <variable mut="var" type="int" name="a" ref_name="_lsiyjvtbvnpmlml"/> <rhs> <literal type="int" value="1" /> </rhs> </declaration> <stream type="std_output"> <operator op="*" type="unset"> <lhs> <variable mut="var" type="int" name="a" ref_name="_lsiyjvtbvnpmlml"/> </lhs> <rhs> <literal type="int" value="42" /> </rhs> </operator> </stream> <return> <literal type="int" value="0" /> </return> </block>'
        parser = PythonUnparser(ET.fromstring(input), True)
        parser.source = ""
        parser.unparse_node(parser.xml)
        self.assertIsNotNone(parser.source)
        self.assertEqual("\n    a: int = 1\n    print(a * 42, end='')\n    return 0\n\n\n", parser.source)

    def test_unparse_assignment(self):
        with open("xml/assignment.xml", "r") as f:
            input = f.read()
        parser = PythonUnparser(ET.fromstring(input), True)
        parser.source = ""
        parser.unparse_node(parser.xml)
        self.assertIsNotNone(parser.source)
        self.assertEqual("C = 30;\n", parser.source)

    def test_unparse_conditional(self):  # TODO test the else-if statements
        with open("xml/conditional.xml", "r") as f:
            input = f.read()

        with open("xml/conditional.out", "r") as f:
            output = f.read()

        parser = PythonUnparser(ET.fromstring(input), True)
        parser.source = ""

        parser.unparse_node(parser.xml)

        self.assertIsNotNone(parser.source)
        self.assertEqual(output, parser.source)

    def test_unparse_loop(self):
        with open("xml/loop.xml", "r") as f:
            input = f.read()

        with open("xml/loop.out", "r") as f:
            output = f.read()

        parser = PythonUnparser(ET.fromstring(input), True)
        parser.source = ""
        parser.unparse_node(parser.xml)
        self.assertIsNotNone(parser.source)
        self.assertEqual(output, parser.source)

    def test_unparse_operation_single(self):
        input = '<operator op="multiplication" type="unset"> <lhs> <variable mut="var" type="int" name="a" ref_name="_lsiyjvtbvnpmlml"/> </lhs> <rhs> <literal type="int" value="42" /> </rhs> </operator>'
        parser = PythonUnparser(ET.fromstring(input), True)
        parser.source = ""
        parser.unparse_node(parser.xml)
        self.assertIsNotNone(parser.source)
        self.assertEqual("a * 42", parser.source)

    def test_unparse_return(self):
        input = '<return> <literal type="int" value="0" /> </return>'
        parser = PythonUnparser(ET.fromstring(input), True)
        parser.source = ""
        parser.unparse_node(parser.xml)
        self.assertIsNotNone(parser.source)
        self.assertEqual("return 0\n", parser.source)

    def test_unparse_unary(self):
        with open("xml/unary.xml", "r") as f:
            input = f.read()

        with open("xml/unary.out", "r") as f:
            output = f.read()

        parser = PythonUnparser(ET.fromstring(input), True)
        parser.source = ""
        parser.unparse_node(parser.xml)
        self.assertIsNotNone(parser.source)
        self.assertEqual(output, parser.source)

    def test_unparse_routine(self):
        input = '<procedure name="main" ref_name="_jemrvvseyj" return_type="int" args="()" ><block><declaration><variable mut="var" type="int" name="a" ref_name="_lsiyjvtbvnpmlml"/><rhs><literal type="int" value="1" /></rhs></declaration><stream type="std_output"><operator op="*" type="unset"><lhs><variable mut="var" type="int" name="a" ref_name="_lsiyjvtbvnpmlml"/></lhs><rhs><literal type="int" value="42" /></rhs></operator></stream><return><literal type="int" value="0" /></return></block></procedure>'
        parser = PythonUnparser(ET.fromstring(input), True)
        parser.source = ""
        parser.unparse_node(parser.xml)
        self.assertIsNotNone(parser.source)
        i = ' ' * parser.indentation
        self.assertEqual("procedure main() returns integer {\n    var integer a = 1;\n    a * 42 -> std_output;\n    return 0;\n}\n\n", parser.source)

    def test_unparse_args(self):
        with open("xml/many_args.xml", 'r') as f:
            input = f.read()
        with open("xml/many_args.out", 'r') as f:
            output = f.read()
        parser = PythonUnparser(ET.fromstring(input), True)
        parser.source = ""
        parser.unparse_node(parser.xml)
        self.assertIsNotNone(parser.source)
        self.assertEqual(output, parser.source)


    def test_unparse_code(self):
        with open("xml/test.xml", "r") as input:
            parser = PythonUnparser(ET.fromstring(input.read()), True)
            parser.unparse()
            self.assertIsNotNone(parser.source)

        with open("input.in", "r") as input:
            i = input.read()
            self.assertEqual(i, parser.source)


if __name__ == "__main__":
    unittest.main()